"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var DOMException_1 = __importDefault(require("../../exception/DOMException"));
/**
 * DOM Token List.
 */
var DOMTokenList = /** @class */ (function () {
    /**
     * Adds class names.
     *
     * @param ownerElement Owner element.
     */
    function DOMTokenList(ownerElement) {
        this._ownerElement = ownerElement;
    }
    /**
     * Get ClassName.
     *
     * @param index Index.
     * */
    DOMTokenList.prototype.item = function (index) {
        this._refreshListFromClass();
        return index >= 0 && this._list[index] ? this._list[index] : null;
    };
    /**
     * Replace Token.
     *
     * @param token Token.
     * @param newToken NewToken.
     */
    DOMTokenList.prototype.replace = function (token, newToken) {
        if (!this.contains(token)) {
            return false;
        }
        var index = this._list.indexOf(token);
        this._list[index] = newToken;
        var value = this._list ? this._list.join(' ') : '';
        this._ownerElement.setAttribute('class', value);
        return true;
    };
    /**
     * Supports.
     *
     * @param token Token.
     */
    DOMTokenList.prototype.supports = function (token) {
        // TODO May IT IN ERROR.
        throw new DOMException_1.default("Failed to execute '".concat(token, "' on 'DOMTokenList': DOMTokenList has no supported tokens."), 'TypeError');
    };
    /**
     * Returns an iterator, allowing you to go through all values of the key/value pairs contained in this object.
     *
     *
     */
    DOMTokenList.prototype.values = function () {
        this._refreshListFromClass();
        return this._list.values();
    };
    /**
     * Returns an iterator, allowing you to go through all key/value pairs contained in this object.
     *
     *
     */
    DOMTokenList.prototype.entries = function () {
        this._refreshListFromClass();
        return this._list.entries();
    };
    /**
     * Executes a provided callback function once for each DOMTokenList element.
     *
     * @param callback
     * @param thisArg
     */
    DOMTokenList.prototype.forEach = function (callback, thisArg) {
        this._refreshListFromClass();
        return this._list.forEach(callback, thisArg);
    };
    /**
     * Returns an iterator, allowing you to go through all keys of the key/value pairs contained in this object.
     *
     */
    DOMTokenList.prototype.keys = function () {
        this._refreshListFromClass();
        return this._list.keys();
    };
    /**
     * Adds class names.
     *
     * @param classNames Class names.
     */
    DOMTokenList.prototype.add = function () {
        var classNames = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            classNames[_i] = arguments[_i];
        }
        this._refreshListFromClass();
        for (var _a = 0, classNames_1 = classNames; _a < classNames_1.length; _a++) {
            var className = classNames_1[_a];
            if (!this._list.includes(className)) {
                if (className.includes(' ')) {
                    throw new DOMException_1.default("Failed to execute 'add' on 'DOMTokenList': The token provided ('".concat(className, "') contains HTML space characters, which are not valid in tokens."));
                }
                this._list.push(className);
            }
        }
        this._list = Array.from(new Set(this._list));
        var value = this._list ? this._list.join(' ') : '';
        this._ownerElement.setAttribute('class', value);
    };
    /**
     * Removes class names.
     *
     * @param classNames Class names.
     */
    DOMTokenList.prototype.remove = function () {
        var classNames = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            classNames[_i] = arguments[_i];
        }
        this._refreshListFromClass();
        for (var _a = 0, classNames_2 = classNames; _a < classNames_2.length; _a++) {
            var className = classNames_2[_a];
            var index = this._list.indexOf(className);
            if (index !== -1) {
                this._list.splice(index, 1);
            }
        }
        var value = this._list ? this._list.join(' ') : '';
        this._ownerElement.setAttribute('class', value);
    };
    /**
     * Check if the list contains a class.
     *
     * @param className Class name.
     * @returns TRUE if it contains.
     */
    DOMTokenList.prototype.contains = function (className) {
        this._refreshListFromClass();
        return this._list.includes(className);
    };
    /**
     * Toggle a class name.
     *
     * @param className A string representing the class name you want to toggle.
     * @param force If included, turns the toggle into a one way-only operation. If set to `false`, then class name will only be removed, but not added. If set to `true`, then class name will only be added, but not removed.
     * @returns A boolean value, `true` or `false`, indicating whether class name is in the list after the call or not.
     */
    DOMTokenList.prototype.toggle = function (className, force) {
        var shouldAdd;
        if (force !== undefined) {
            shouldAdd = force;
        }
        else {
            shouldAdd = !this.contains(className);
        }
        if (shouldAdd) {
            this.add(className);
            return true;
        }
        this.remove(className);
        return false;
    };
    /**
     * Refresh list from class.
     */
    DOMTokenList.prototype._refreshListFromClass = function () {
        var attr = this._ownerElement.getAttribute('class');
        this._list = attr ? Array.from(new Set(attr.split(' '))) : [];
    };
    Object.defineProperty(DOMTokenList.prototype, "value", {
        /**
         * Get Value.
         */
        get: function () {
            this._refreshListFromClass();
            return this._list ? this._list.join(' ') : '';
        },
        /**
         * Set Value.
         */
        set: function (value) {
            this._ownerElement.setAttribute('class', value);
            this._list = value ? Array.from(new Set(value.split(' '))) : [];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DOMTokenList.prototype, "length", {
        /**
         * Get Length.
         */
        get: function () {
            this._refreshListFromClass();
            return this._list.length;
        },
        enumerable: false,
        configurable: true
    });
    return DOMTokenList;
}());
exports.default = DOMTokenList;
//# sourceMappingURL=DOMTokenList.js.map